import { AxiosRequestConfig } from "axios";
import { ItemResult, ListResult } from "./api-client.js";
import type { ItemMethod, ListMethod, Method, TypeDiscriminatorToType } from "./metadata.js";
type PromiseOrSync<T> = T | Promise<T>;
type EndpointSpec = Method | `/${string}/${"list" | "get" | "count" | "save" | "bulkSave" | "delete" | string}`;
type EndpointMock<TEndpoint extends EndpointSpec> = (request: AxiosRequestConfig) => PromiseOrSync<TEndpoint extends ListMethod | `/${string}/${"list"}` ? ListResult : TEndpoint extends `/${string}/${"count"}` ? ItemResult<number> : TEndpoint extends ItemMethod ? ItemResult<TypeDiscriminatorToType<TEndpoint["return"]["type"]>> : ItemResult | ListResult>;
/** Mock a coalesce API endpoint, providing your own implementation for the API request.
 
 * Example:
 * ```
 * mockEndpoint(
      "/Course/list",
      (req) => ({
        wasSuccessful: true,
        list: [
          { courseId: 1, title: "CS 101" },
          { courseId: 2, title: "CS 102" },
        ],
      })
    )
 * ```
 *
 * Example:
 * ```
 * const fn = mockEndpoint(
      new Student().$metadata.methods.getName,
      vitest.fn((req) => ({
        wasSuccessful: true,
        object: "Steve",
      }))
    )
    // Do testing ...
    expect(fn).toHaveBeenCalledOnce();
    fn.destroy();
 * ```
 */
export declare function mockEndpoint<TEndpoint extends EndpointSpec, TMock extends EndpointMock<TEndpoint>>(endpoint: TEndpoint, mock: TMock): TMock & {
    /** Tear down the mock, removing the handler from the global axios adapter. */
    destroy(): void;
};
export {};
